@@ -490,6 +490,7 @@ typedef struct _swConnection
     uint8_t ssl_want_write;
     uint8_t http_upgrade;
     uint8_t http2_stream;
+    uint8_t skip_recv;
     //--------------------------------------------------------------
     /**
      * server is actively close the connection
@@ -1558,7 +1559,7 @@ struct _swProcessPool
      * reloading
      */
     uint8_t reloading;
-    uint8_t reload_flag;
+    uint8_t reload_init;
     uint8_t dispatch_mode;
     uint8_t ipc_mode;
 
