@@ -410,6 +410,7 @@ static int swClient_inet_addr(swClient *cli, char *host, int port)
     {
         if (swoole_gethostbyname(cli->_sock_domain, host, s_addr) < 0)
         {
+            SwooleG.error = SW_ERROR_DNSLOOKUP_RESOLVE_FAILED;
             return SW_ERR;
         }
     }
@@ -453,6 +454,7 @@ static int swClient_close(swClient *cli)
 {
     int fd = cli->socket->fd;
     assert(fd != 0);
+
 #ifdef SW_USE_OPENSSL
     if (cli->open_ssl && cli->ssl_context)
     {
@@ -1072,7 +1074,7 @@ static int swClient_onPackage(swConnection *conn, char *data, uint32_t length)
 {
     swClient *cli = (swClient *) conn->object;
     cli->onReceive(conn->object, data, length);
-    return SW_OK;
+    return cli->socket->close_wait ? SW_ERR : SW_OK;
 }
 
 static int swClient_onStreamRead(swReactor *reactor, swEvent *event)
