@@ -1511,6 +1511,10 @@ static void swServer_signal_hanlder(int sig)
         {
             break;
         }
+        if (SwooleG.serv->factory_mode == SW_MODE_SINGLE)
+        {
+            break;
+        }
         pid = waitpid(-1, &status, WNOHANG);
         if (pid > 0 && pid == SwooleGS->manager_pid)
         {
@@ -1530,8 +1534,12 @@ static void swServer_signal_hanlder(int sig)
     case SIGUSR2:
         if (SwooleG.serv->factory_mode == SW_MODE_SINGLE)
         {
+            if (SwooleGS->event_workers.reloading)
+            {
+                break;
+            }
             SwooleGS->event_workers.reloading = 1;
-            SwooleGS->event_workers.reload_flag = 0;
+            SwooleGS->event_workers.reload_init = 0;
         }
         else
         {
