@@ -20,6 +20,8 @@
 #ifdef SW_USE_OPENSSL
 
 #include <openssl/crypto.h>
+#include <openssl/x509.h>
+#include <openssl/x509v3.h>
 
 static int openssl_init = 0;
 static pthread_mutex_t *lock_array;
@@ -384,6 +386,175 @@ int swSSL_set_client_certificate(SSL_CTX *ctx, char *cert_file, int depth)
     return SW_OK;
 }
 
+int swSSL_set_capath(swSSL_option *cfg, SSL_CTX *ctx)
+{
+    if (cfg->cafile || cfg->capath)
+    {
+        if (!SSL_CTX_load_verify_locations(ctx, cfg->cafile, cfg->capath))
+        {
+            return SW_ERR;
+        }
+    }
+    else
+    {
+        if (!SSL_CTX_set_default_verify_paths(ctx))
+        {
+            swWarn("Unable to set default verify locations and no CA settings specified.");
+            return SW_ERR;
+        }
+    }
+
+    if (cfg->verify_depth > 0)
+    {
+        SSL_CTX_set_verify_depth(ctx, cfg->verify_depth);
+    }
+
+    return SW_OK;
+}
+
+#ifndef X509_CHECK_FLAG_ALWAYS_CHECK_SUBJECT
+static int swSSL_check_name(char *name, ASN1_STRING *pattern)
+{
+    char *s, *end;
+    size_t slen, plen;
+
+    s = name;
+    slen = strlen(name);
+
+    uchar *p = ASN1_STRING_data(pattern);
+    plen = ASN1_STRING_length(pattern);
+
+    if (slen == plen && strncasecmp(s, (char*) p, plen) == 0)
+    {
+        return SW_OK;
+    }
+
+    if (plen > 2 && p[0] == '*' && p[1] == '.')
+    {
+        plen -= 1;
+        p += 1;
+
+        end = s + slen;
+        s = swoole_strlchr(s, end, '.');
+
+        if (s == NULL)
+        {
+            return SW_ERR;
+        }
+
+        slen = end - s;
+
+        if (plen == slen && strncasecmp(s, (char*) p, plen) == 0)
+        {
+            return SW_OK;
+        }
+    }
+    return SW_ERR;
+}
+#endif
+
+int swSSL_check_host(swConnection *conn, char *tls_host_name)
+{
+    X509 *cert = SSL_get_peer_certificate(conn->ssl);
+    if (cert == NULL)
+    {
+        return SW_ERR;
+    }
+
+#ifdef X509_CHECK_FLAG_ALWAYS_CHECK_SUBJECT
+    /* X509_check_host() is only available in OpenSSL 1.0.2+ */
+    if (X509_check_host(cert, tls_host_name, strlen(tls_host_name), 0, NULL) != 1)
+    {
+        swWarn("X509_check_host(): no match");
+        goto failed;
+    }
+    goto found;
+#else
+    int n, i;
+    X509_NAME *sname;
+    ASN1_STRING *str;
+    X509_NAME_ENTRY *entry;
+    GENERAL_NAME *altname;
+    STACK_OF(GENERAL_NAME) *altnames;
+
+    /*
+     * As per RFC6125 and RFC2818, we check subjectAltName extension,
+     * and if it's not present - commonName in Subject is checked.
+     */
+    altnames = X509_get_ext_d2i(cert, NID_subject_alt_name, NULL, NULL);
+
+    if (altnames)
+    {
+        n = sk_GENERAL_NAME_num(altnames);
+
+        for (i = 0; i < n; i++)
+        {
+            altname = sk_GENERAL_NAME_value(altnames, i);
+
+            if (altname->type != GEN_DNS)
+            {
+                continue;
+            }
+
+            str = altname->d.dNSName;
+            swTrace("SSL subjectAltName: \"%*s\"", ASN1_STRING_length(str), ASN1_STRING_data(str));
+
+            if (swSSL_check_name(tls_host_name, str) == SW_OK)
+            {
+                swTrace("SSL subjectAltName: match");
+                GENERAL_NAMES_free(altnames);
+                goto found;
+            }
+        }
+
+        swTrace("SSL subjectAltName: no match.");
+        GENERAL_NAMES_free(altnames);
+        goto failed;
+    }
+
+    /*
+     * If there is no subjectAltName extension, check commonName
+     * in Subject.  While RFC2818 requires to only check "most specific"
+     * CN, both Apache and OpenSSL check all CNs, and so do we.
+     */
+    sname = X509_get_subject_name(cert);
+
+    if (sname == NULL)
+    {
+        goto failed;
+    }
+
+    i = -1;
+    for (;;)
+    {
+        i = X509_NAME_get_index_by_NID(sname, NID_commonName, i);
+
+        if (i < 0)
+        {
+            break;
+        }
+
+        entry = X509_NAME_get_entry(sname, i);
+        str = X509_NAME_ENTRY_get_data(entry);
+
+        swTrace("SSL commonName: \"%*s\"", ASN1_STRING_length(str), ASN1_STRING_data(str));
+
+        if (swSSL_check_name(tls_host_name, str) == SW_OK)
+        {
+            swTrace("SSL commonName: match");
+            goto found;
+        }
+    }
+    swTrace("SSL commonName: no match");
+#endif
+
+    failed: X509_free(cert);
+    return SW_ERR;
+
+    found: X509_free(cert);
+    return SW_OK;
+}
+
 int swSSL_verify(swConnection *conn, int allow_self_signed)
 {
     int err = SSL_get_verify_result(conn->ssl);
@@ -502,7 +673,7 @@ int swSSL_accept(swConnection *conn)
     {
         return SW_ERROR;
     }
-    swWarn("SSL_do_handshake() failed. Error: [%ld].", err);
+    swWarn("SSL_do_handshake() failed. Error: [%ld|%d].", err, errno);
     return SW_ERROR;
 }
 
@@ -549,12 +720,16 @@ int swSSL_sendfile(swConnection *conn, int fd, off_t *offset, size_t size)
         ret = swSSL_send(conn, buf, n);
         if (ret < 0)
         {
-            swSysError("write() failed.");
+            if (swConnection_error(errno) == SW_ERROR)
+            {
+                swSysError("write() failed.");
+            }
         }
         else
         {
             *offset += ret;
         }
+        swTraceLog(SW_TRACE_REACTOR, "fd=%d, readn=%d, n=%d, ret=%d", fd, readn, n, ret);
         return ret;
     }
     else
