@@ -143,6 +143,9 @@ static void php_swoole_check_aio()
     {
         php_swoole_check_reactor();
         swAio_init();
+    }
+    if (!SwooleAIO.callback)
+    {
         SwooleAIO.callback = php_swoole_aio_onComplete;
     }
 }
@@ -877,11 +880,6 @@ PHP_FUNCTION(swoole_async_set)
         convert_to_boolean(v);
         SWOOLE_G(display_errors) = 0;
     }
-    if (php_swoole_array_get_value(vht, "aio_chunk_size", v))
-    {
-        convert_to_string(v);
-        SwooleG.dns_server_v4 = sw_strndup(Z_STRVAL_P(v), Z_STRLEN_P(v));
-    }
     if (php_swoole_array_get_value(vht, "socket_dontwait", v))
     {
         convert_to_boolean(v);
