@@ -467,12 +467,49 @@ int swoole_http_parse_form_data(http_context *ctx, const char *boundary_str, int
     return SW_OK;
 }
 
+static inline char* http_trim_double_quote_str(char *ptr, int *len)
+{
+    int i;
+    char *tmp = ptr;
+
+    //ltrim('"')
+    for (i = 0; i < *len; i++)
+    {
+        if (tmp[i] == '"')
+        {
+            (*len)--;
+            tmp++;
+            continue;
+        }
+        else
+        {
+            break;
+        }
+    }
+    //rtrim('"')
+    for (i = (*len) - 1; i > 0; i--)
+    {
+        if (tmp[i] == '"')
+        {
+            (*len)--;
+            continue;
+        }
+        else
+        {
+            break;
+        }
+    }
+    tmp[(*len)] = 0;
+    return tmp;
+}
+
 static void http_parse_cookie(zval *array, const char *at, size_t length)
 {
     char keybuf[SW_HTTP_COOKIE_KEYLEN];
     char valbuf[SW_HTTP_COOKIE_VALLEN];
     char *_c = (char *) at;
 
+    char *_value;
     int klen = 0;
     int vlen = 0;
     int state = -1;
@@ -504,10 +541,11 @@ static void http_parse_cookie(zval *array, const char *at, size_t length)
             }
             memcpy(valbuf, (char *) at + j, vlen);
             valbuf[vlen] = 0;
-            vlen = php_url_decode(valbuf, vlen);
+            _value = http_trim_double_quote_str(valbuf, &vlen);
+            vlen = php_url_decode(_value, vlen);
             if (klen > 1)
             {
-                sw_add_assoc_stringl_ex(array, keybuf, klen, valbuf, vlen, 1);
+                sw_add_assoc_stringl_ex(array, keybuf, klen, _value, vlen, 1);
             }
             j = i + 1;
             state = -1;
@@ -542,11 +580,12 @@ static void http_parse_cookie(zval *array, const char *at, size_t length)
             return;
         }
         memcpy(valbuf, (char *) at + j, vlen);
-        valbuf[vlen] = 0;;
-        vlen = php_url_decode(valbuf, vlen);
+        valbuf[vlen] = 0;
+        _value = http_trim_double_quote_str(valbuf, &vlen);
+        vlen = php_url_decode(_value, vlen);
         if (klen > 1)
         {
-            sw_add_assoc_stringl_ex(array, keybuf, klen, valbuf, vlen, 1);
+            sw_add_assoc_stringl_ex(array, keybuf, klen, _value, vlen, 1);
         }
     }
 }
@@ -1588,8 +1627,6 @@ static PHP_METHOD(swoole_http_server, start)
     serv->listen_list->open_eof_check = 0;
     serv->listen_list->open_length_check = 0;
 
-    serv->ptr2 = getThis();
-
     //for is_uploaded_file and move_uploaded_file
     ALLOC_HASHTABLE(SG(rfc1867_uploaded_files));
     zend_hash_init(SG(rfc1867_uploaded_files), 8, NULL, NULL, 0);
@@ -2594,6 +2631,8 @@ static PHP_METHOD(swoole_http_response, __destruct)
         }
         else
         {
+            context->response.status = 500;
+
             zval *zobject = getThis();
             zval *retval = NULL;
             sw_zend_call_method_with_0_params(&zobject, swoole_http_response_class_entry_ptr, NULL, "end", &retval);
